use actix_web::{http::header::LOCATION, web, HttpResponse, Responder};
use chrono::{NaiveDate, NaiveTime};
use garde::Validate;
use serde::Deserialize;
use sqlx::PgPool;

use crate::{
    endpoints::availability::find_adjacend_availability,
    models::{Availability, AvailabilityChangeset, AvailabilityContext, AvailabilityTime, User},
    utils::{self, ApplicationError},
};

#[derive(Deserialize)]
pub struct AvailabillityForm {
    pub date: NaiveDate,
    pub from: Option<NaiveTime>,
    pub till: Option<NaiveTime>,
    pub comment: Option<String>,
}

#[actix_web::post("/availabillity/new")]
pub async fn post(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    form: web::Form<AvailabillityForm>,
) -> Result<impl Responder, ApplicationError> {
    let existing_availabilities =
        Availability::read_by_user_and_date(pool.get_ref(), user.id, &form.date).await?;
    let context = AvailabilityContext {
        existing_availabilities: existing_availabilities.clone(),
    };

    let time = if form.from.is_some() && form.till.is_some() {
        AvailabilityTime::Temporarily(form.from.unwrap(), form.till.unwrap())
    } else {
        AvailabilityTime::WholeDay
    };

    let mut changeset = AvailabilityChangeset {
        time,
        comment: form.comment.clone(),
    };

    if let Err(e) = changeset.validate_with(&context) {
        return Ok(HttpResponse::BadRequest().body(e.to_string()));
    };

    if let Some(a) = find_adjacend_availability(&changeset, None, &existing_availabilities) {
        let (changeset_start, changeset_end) = changeset.time.time_tuple_opt().unwrap();
        let (adjacent_start, adjacent_end) = changeset.time.time_tuple_opt().unwrap();

        if adjacent_end == changeset_start {
            changeset.time = AvailabilityTime::Temporarily(adjacent_start, changeset_end);
        }

        if adjacent_start == changeset_end {
            changeset.time = AvailabilityTime::Temporarily(changeset_start, adjacent_end);
        }

        Availability::update(pool.get_ref(), a.id, changeset).await?;
    } else {
        Availability::create(pool.get_ref(), user.id, form.date, changeset).await?;
    }

    let url = utils::get_return_url_for_date(&form.date);
    Ok(HttpResponse::Found()
        .insert_header((LOCATION, url.clone()))
        .insert_header(("HX-LOCATION", url))
        .finish())
}
