use actix_web::{web, HttpResponse, Responder};
use askama::Template;
use sqlx::PgPool;

use crate::{
    endpoints::IdPath,
    filters,
    models::{Role, User},
    utils::ApplicationError,
};

#[derive(Template, Debug)]
#[template(path = "user/overview_locked_td.html")]
struct OverviewPartialLockedTemplate {
    u: User,
    is_oob: bool,
}

#[derive(Template, Debug)]
#[template(path = "user/overview_buttons_td.html")]
struct OverviewPartialButtonsTemplate {
    u: User,
}

#[actix_web::put("/users/{id}/lock")]
pub async fn put_lock_user(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
) -> Result<impl Responder, ApplicationError> {
    handle_lock_state_for_user(user, pool, path, true).await
}

#[actix_web::put("/users/{id}/unlock")]
pub async fn put_unlock_user(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
) -> Result<impl Responder, ApplicationError> {
    handle_lock_state_for_user(user, pool, path, false).await
}

async fn handle_lock_state_for_user(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
    lock_state: bool,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::AreaManager && user.role != Role::Admin {
        return Err(ApplicationError::Unauthorized);
    }

    if user.id == path.id {
        return Ok(HttpResponse::BadRequest().finish());
    }

    let Some(mut user_in_db) = User::read_by_id(pool.get_ref(), path.id).await? else {
        return Ok(HttpResponse::NotFound().finish());
    };

    if user.role == Role::AreaManager
        && (user.area_id != user_in_db.area_id || user_in_db.role == Role::Admin)
    {
        return Err(ApplicationError::Unauthorized);
    }

    if user_in_db.locked != lock_state {
        User::update_locked(pool.get_ref(), user_in_db.id, lock_state).await?;
        user_in_db.locked = lock_state;
    }

    let buttons = OverviewPartialButtonsTemplate {
        u: user_in_db.clone(),
    };

    let locked_oob = OverviewPartialLockedTemplate {
        u: user_in_db,
        is_oob: true,
    };

    let mut body = buttons.render()?;
    body.push_str(&locked_oob.render()?);

    Ok(HttpResponse::Ok().body(body))
}

// TODO: Tests schreiben
// #[cfg(test)]
// mod tests {
//     use crate::utils::test_helper::{
//         assert_snapshot, read_body, test_put, DbTestContext, RequestConfig, StatusCode,
//     };
//     use brass_macros::db_test;
//
//     #[db_test]
//     async fn user_can_toggle_subscription_for_himself(context: &DbTestContext) {
//         let app = context.app().await;
//
//         let unsubscribe_config = RequestConfig::new("/users/1/unsubscribeNotifications");
//         let unsubscribe_response =
//             test_put::<_, _, String>(&context.db_pool, &app, &unsubscribe_config, None).await;
//
//         assert_eq!(StatusCode::OK, unsubscribe_response.status());
//
//         let unsubscribe_body = read_body(unsubscribe_response).await;
//         assert_snapshot!(unsubscribe_body);
//
//         let subscribe_config = RequestConfig::new("/users/1/subscribeNotifications");
//         let subscribe_response =
//             test_put::<_, _, String>(&context.db_pool, &app, &subscribe_config, None).await;
//
//         assert_eq!(StatusCode::OK, subscribe_response.status());
//
//         let subscribe_body = read_body(subscribe_response).await;
//         assert_snapshot!(subscribe_body);
//     }
//
//     #[db_test]
//     async fn user_cant_toggle_subscription_for_someone_else(context: &DbTestContext) {
//         let app = context.app().await;
//
//         let unsubscribe_config = RequestConfig::new("/users/3/unsubscribeNotifications");
//         let unsubscribe_response =
//             test_put::<_, _, String>(&context.db_pool, &app, &unsubscribe_config, None).await;
//
//         assert_eq!(StatusCode::UNAUTHORIZED, unsubscribe_response.status());
//     }
// }
